@@ -13,7 +13,7 @@ import numpy as np
 from .operator_converters import constants
 from ._parse import parse_sklearn_api_model, parse_onnx_api_model
 from ._topology import convert as topology_converter
-from ._utils import torch_installed, lightgbm_installed, xgboost_installed, onnx_runtime_installed
+from ._utils import torch_installed, lightgbm_installed, xgboost_installed
 from .exceptions import MissingConverter, MissingBackend
 from .supported import backends
 
@@ -36,47 +36,35 @@ def _supported_backend_check(backend):
     """
     Function used to check whether the specified backend is supported or not.
     """
-    if backend not in backends:
+    if backend is None:
         raise MissingBackend("Backend: {}".format(backend))
 
 
-def _supported_model_format_backend_mapping_check(model, backend):
+def _supported_backend_check_config(model, backend, extra_config):
     """
-    Function used to check whether the specified backend/input model format is supported or not.
+    Function used to check whether the specified backend and configuration pair is supported or not.
     """
-    if _is_onnx_model(model):
-        assert onnx_runtime_installed()
-        import onnx
+    assert torch_installed(), "To use Hummingbird you need to install torch."
+    import onnx
+    import torch
 
-        if not backend == onnx.__name__:
-            raise RuntimeError("Hummingbird currently support conversion of ONNX(-ML) models only into ONNX.")
-    else:
-        assert torch_installed()
-        import torch
+    if backend is torch.jit.__name__ and constants.TEST_INPUT not in extra_config:
+        raise RuntimeError("Backend {} requires test inputs. Please pass some test input to the convert.".format(backend))
 
-        if not backend == torch.__name__ and not backend == "py" + torch.__name__:
-            raise RuntimeError(
-                "Hummingbird currently support conversion of XGBoost / LightGBM / Sklearn models only into PyTorch."
-            )
+    if backend is onnx.__name__:
+        if constants.ONNX_INITIAL_TYPES not in extra_config and constants.TEST_INPUT not in extra_config:
+            raise RuntimeError("Cannot generate test input data for ONNX. Either pass some input data or the initial_types")
+
+    if _is_onnx_model(model) and constants.ONNX_INITIAL_TYPES not in extra_config and constants.TEST_INPUT not in extra_config:
+        raise RuntimeError(
+            "Cannot extract number of input features from the ONNX. Either pass some input data or the initial_types"
+        )
 
 
-def _convert_sklearn(model, test_input=None, extra_config={}):
+def _convert_sklearn(model, backend, test_input, device, extra_config={}):
     """
-    This function converts the specified *scikit-learn* (API) model into its [PyTorch] counterpart.
-    The supported operators can be found at `hummingbird.ml.supported`.
-    [PyTorch]: https://pytorch.org/
-
-    Args:
-        model: A scikit-learn model
-        test_input: some input data used to trace the model execution
-        extra_config: Extra configurations to be used by the individual operator converters.
-                      The set of supported extra configurations can be found at `hummingbird.ml.supported`
-
-    Examples:
-        >>> pytorch_model = _convert_sklearn(sklearn_model)
-
-    Returns:
-        A model implemented in *PyTorch*, which is equivalent to the input *scikit-learn* model
+    This function converts the specified *scikit-learn* (API) model into its *backend* counterpart.
+    The supported operators and backends can be found at `hummingbird.ml.supported`.
     """
     assert model is not None
     assert torch_installed(), "To use Hummingbird you need to install torch."
@@ -89,52 +77,26 @@ def _convert_sklearn(model, test_input=None, extra_config={}):
     topology = parse_sklearn_api_model(model)
 
     # Convert the Topology object into a PyTorch model.
-    hb_model = topology_converter(topology, torch.__name__, extra_config=extra_config)
+    hb_model = topology_converter(topology, backend, device, extra_config=extra_config)
     return hb_model
 
 
-def _convert_lightgbm(model, test_input=None, extra_config={}):
+def _convert_lightgbm(model, backend, test_input, device, extra_config={}):
     """
-    This function is used to generate a [PyTorch] model from a given input [LightGBM] model.
+    This function is used to generate a *backend* model from a given input [LightGBM] model.
     [LightGBM]: https://lightgbm.readthedocs.io/
-    [PyTorch]: https://pytorch.org/
-
-    Args:
-        model: A LightGBM model (trained using the scikit-learn API)
-        test_input: Some input data that will be used to trace the model execution
-        extra_config: Extra configurations to be used by the individual operator converters.
-                      The set of supported extra configurations can be found at `hummingbird.ml.supported`
-
-    Examples:
-        >>> pytorch_model = _convert_lightgbm(lgbm_model)
-
-    Returns:
-        A *PyTorch* model which is equivalent to the input *LightGBM* model
     """
     assert (
         lightgbm_installed()
     ), "To convert LightGBM models you need to install LightGBM (or `pip install hummingbird-ml[extra]`)."
 
-    return _convert_sklearn(model, test_input, extra_config)
+    return _convert_sklearn(model, backend, test_input, device, extra_config)
 
 
-def _convert_xgboost(model, test_input, extra_config={}):
+def _convert_xgboost(model, backend, test_input, device, extra_config={}):
     """
-    This function is used to generate a [PyTorch] model from a given input [XGBoost] model.
-    [PyTorch]: https://pytorch.org/
+    This function is used to generate a *backend* model from a given input [XGBoost] model.
     [XGBoost]: https://xgboost.readthedocs.io/
-
-    Args:
-        model: A XGBoost model (trained using the scikit-learn API)
-        test_input: Some input data used to trace the model execution
-        extra_config: Extra configurations to be used by the individual operator converters.
-                      The set of supported extra configurations can be found at `hummingbird.ml.supported`
-
-    Examples:
-        >>> pytorch_model = _convert_xgboost(xgb_model, [], extra_config={"n_features":200})
-
-    Returns:
-        A *PyTorch* model which is equivalent to the input *XGBoost* model
     """
     assert (
         xgboost_installed()
@@ -159,37 +121,16 @@ def _convert_xgboost(model, test_input, extra_config={}):
                 "XGBoost converter is not able to infer the number of input features.\
                     Please pass some test_input to the converter."
             )
-    return _convert_sklearn(model, test_input, extra_config)
+    return _convert_sklearn(model, backend, test_input, device, extra_config)
 
 
-def _convert_onnxml(model, test_input=None, extra_config={}):
+def _convert_onnxml(model, backend, test_input, device, extra_config={}):
     """
-    This function converts the specified [ONNX-ML] model into its [ONNX] counterpart.
+    This function converts the specified [ONNX-ML] model into its *backend* counterpart.
     The supported operators can be found at `hummingbird.ml.supported`.
-    The ONNX-ML converter requires either a test_input of a the initial types set through the exta_config parameter.
-    [ONNX-ML]: https://github.com/onnx/onnx/blob/master/docs/Operators-ml.md
-    [ONNX]: https://github.com/onnx/onnx/blob/master/docs/Operators.md
-
-    Args:
-        model: A model containing ONNX-ML operators
-        test_input: Some input data used to trace the model execution.
-                    For the ONNX backend the test_input size is supposed to be as large as the expected batch size.
-        extra_config: Extra configurations to be used by the individual operator converters.
-                      The set of supported extra configurations can be found at `hummingbird.ml.supported`
-
-    Examples:
-        extra_config = {}
-        extra_config[constans.ONNX_INITIAL_TYPES] =[('input', FloatTensorType([1, 20])]
-        >>> onnx_model = _convert_onnxml(onnx_ml_model, None, extra_config)
-
-    Returns:
-        A model containing only *ONNX* operators. The mode is equivalent to the input *ONNX-ML* model
     """
     assert model is not None
     assert torch_installed(), "To use Hummingbird you need to install torch."
-    assert (
-        onnx_runtime_installed()
-    ), "To use the onnxml converter you need to install onnxruntime (or `pip install hummingbird-ml[onnx]`)."
 
     import onnx
 
@@ -200,52 +141,52 @@ def _convert_onnxml(model, test_input=None, extra_config={}):
     if constants.ONNX_INITIAL_TYPES in extra_config:
         initial_types = extra_config[constants.ONNX_INITIAL_TYPES]
 
-    assert (
-        test_input is not None and len(test_input) > 0
-    ) or initial_types is not None, "Cannot generate test input data. Either pass some input data or the initial_types"
-
     # Generate some test input if necessary.
     if test_input is None:
-        assert (
-            not initial_types[0][1].shape is None
-        ), "Cannot generate test input data. Initial_types do not contain shape information."
-        assert len(initial_types[0][1].shape) == 2, "Hummingbird currently support only inputs with len(shape) == 2."
+        if backend == onnx.__name__:
+            assert (
+                initial_types is not None and not initial_types[0][1].shape is None
+            ), "Cannot generate test input data. Initial_types do not contain shape information."
+            assert len(initial_types[0][1].shape) == 2, "Hummingbird currently support only inputs with len(shape) == 2."
 
-        from onnxconverter_common.data_types import FloatTensorType, Int32TensorType
+            from onnxconverter_common.data_types import FloatTensorType, Int32TensorType
 
-        test_input = np.random.rand(initial_types[0][1].shape[0], initial_types[0][1].shape[1])
-        extra_config[constants.N_FEATURES] = initial_types[0][1].shape[1]
-        if type(initial_types[0][1]) is FloatTensorType:
-            test_input = np.array(test_input, dtype=np.float32)
-        elif type(initial_types[0][1]) is Int32TensorType:
-            test_input = np.array(test_input, dtype=np.int32)
-        else:
-            raise RuntimeError(
-                "Type {} not supported. Please fill an issue on https://github.com/microsoft/hummingbird/.".format(
-                    type(initial_types[0][1])
+            test_input = np.random.rand(initial_types[0][1].shape[0], initial_types[0][1].shape[1])
+            extra_config[constants.N_FEATURES] = initial_types[0][1].shape[1]
+            if type(initial_types[0][1]) is FloatTensorType:
+                test_input = np.array(test_input, dtype=np.float32)
+            elif type(initial_types[0][1]) is Int32TensorType:
+                test_input = np.array(test_input, dtype=np.int32)
+            else:
+                raise RuntimeError(
+                    "Type {} not supported. Please fill an issue on https://github.com/microsoft/hummingbird/.".format(
+                        type(initial_types[0][1])
+                    )
                 )
-            )
     else:
         extra_config[constants.N_FEATURES] = np.array(test_input).shape[1]
-    extra_config[constants.ONNX_TEST_INPUT] = test_input
+    extra_config[constants.TEST_INPUT] = test_input
+
+    # Set the initializers. Some converter requires the access to initializers.
+    initializers = {} if model.graph.initializer is None else {in_.name: in_ for in_ in model.graph.initializer}
+    extra_config[constants.ONNX_INITIALIZERS] = initializers
 
     # Parse ONNX model as our internal data structure (i.e., Topology).
     topology = parse_onnx_api_model(model)
 
     # Convert the Topology object into a PyTorch model.
-    hb_model = topology_converter(topology, onnx.__name__, extra_config=extra_config)
+    hb_model = topology_converter(topology, backend, device, extra_config=extra_config)
     return hb_model
 
 
-def convert(model, backend, test_input=None, extra_config={}):
+def convert(model, backend, test_input=None, device="cpu", extra_config={}):
     """
     This function converts the specified input *model* into an implementation targeting *backend*.
     *Convert* supports [Sklearn], [LightGBM], [XGBoost] and [ONNX] models.
     For *LightGBM* and *XGBoost* currently only the Sklearn API is supported.
-    For *Sklearn*, *LightGBM* and *XGBoost* currently only the *torch* backend is supported.
-    For *ONNX* currently only the *onnx* backend is supported. For ONNX models, Hummingbird behave as a model
-    rewriter converting [ONNX-ML] into [ONNX operators].
     The detailed list of models and backends can be found at `hummingbird.ml.supported`.
+    The *onnx* backend requires either a test_input of a the initial types set through the exta_config parameter.
+    The *torch.jit* backend requires a test_input.
     [Sklearn]: https://scikit-learn.org/
     [LightGBM]: https://lightgbm.readthedocs.io/
     [XGBoost]: https://xgboost.readthedocs.io/
@@ -256,8 +197,10 @@ def convert(model, backend, test_input=None, extra_config={}):
     Args:
         model: An input model
         backend: The target for the conversion
-        test_input: some input data used to trace the model execution.
+        test_input: Some input data used to trace the model execution.
                     For the ONNX backend the test_input size is supposed to be as large as the expected batch size.
+        device: The target device the model should be run. This parameter is only used by the *torch** backends, and
+                the devices supported are the one supported by PyTorch, i.e., 'cpu' or 'cuda'.
         extra_config: Extra configurations to be used by the individual operator converters.
                       The set of supported extra configurations can be found at `hummingbird.ml.supported`
 
@@ -269,17 +212,28 @@ def convert(model, backend, test_input=None, extra_config={}):
     """
     assert model is not None
 
+    # We destroy extra_config during conversion, we create a copy here.
+    extra_config = deepcopy(extra_config)
+
+    # Add test input as extra configuration for conversion.
+    if test_input is not None and constants.TEST_INPUT not in extra_config:
+        extra_config[constants.TEST_INPUT] = test_input
+
+    # We do some normalization on backends.
     backend = backend.lower()
+    backend = backends[backend]
+
+    # Check whether we actually support the backend.
     _supported_backend_check(backend)
-    _supported_model_format_backend_mapping_check(model, backend)
+    _supported_backend_check_config(model, backend, extra_config)
 
     if type(model) in xgb_operator_list:
-        return _convert_xgboost(model, test_input, extra_config)
+        return _convert_xgboost(model, backend, test_input, device, extra_config)
 
     if type(model) in lgbm_operator_list:
-        return _convert_lightgbm(model, test_input, extra_config)
+        return _convert_lightgbm(model, backend, test_input, device, extra_config)
 
     if _is_onnx_model(model):
-        return _convert_onnxml(model, test_input, extra_config)
+        return _convert_onnxml(model, backend, test_input, device, extra_config)
 
-    return _convert_sklearn(model, test_input, extra_config)
+    return _convert_sklearn(model, backend, test_input, device, extra_config)
