@@ -14,20 +14,38 @@ BASE_PREDICTION = "base_prediction"
 LEARNING_RATE = "learning_rate"
 """Learning Rate"""
 
+POST_TRANSFORM = "post_transform"
+"""Post transform for tree inference"""
+
+SIGMOID = "LOGISTIC"
+"""Sigmoid post transform"""
+
+SOFTMAX = "SOFTMAX"
+"""Softmax post transform"""
+
+TWEEDIE = "TWEEDIE"
+"""Tweedie post transform"""
+
 GET_PARAMETERS_FOR_TREE_TRAVERSAL = "get_parameters_for_tree_trav"
 """Which function to use to extract the parameters for the tree traversal strategy"""
 
 REORDER_TREES = "reorder_trees"
 """Whether to reorder trees in multiclass tasks"""
 
+ONNX_INITIALIZERS = "onnx_initializers"
+"""The initializers of the onnx model"""
+
 ONNX_INPUTS = "onnx_inputs"
-"""The inputs to the onnx model"""
+"""The input of the onnx model"""
 
-ONNX_TEST_INPUT = "onnx_test_input"
-"""The test input data for the onnx model"""
+TEST_INPUT = "test_input"
+"""The test input data for models that need to be traced"""
 
-OFFSET = "offset_"
+NUM_TREES = "n_trees"
+"""Number of trees composing an ensemble"""
+
+OFFSET = "offset"
 """offset of the sklearn anomaly detection implementation"""
 
-MAX_SAMPLES = "max_samples_"
+MAX_SAMPLES = "max_samples"
 """max_samples of sklearn isolation forest implementation"""
